@@ -3,7 +3,7 @@
 """This module is a stub for classes related to vulnerability exposure scores.
 
 Copyright:
-    (c) 2022 Illumio
+    © 2022 Illumio
 
 License:
     Apache2, see LICENSE for more details.
@@ -16,3 +16,8 @@ from illumio.util import MutableObject
 @dataclass
 class Vulnerability(MutableObject):
     score: int = None
+
+
+__all__ = [
+    'Vulnerability',
+]
